// Copyright 2020 Igor Tolmachev, ITSamples.com. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License"); /
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
// ==============================================================================

package com.itsamples.telnet;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.content.SharedPreferences;
import android.content.res.Configuration;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;

public class HistoryActivity extends Activity implements CustomMenu.OnMenuItemSelectedListener
{
    public static final int MENU_ITEM_CLEAR  = 1;

    private CustomMenu mMenu;
    private ListView mListCommands;
    private LayoutInflater mInflater;
    final public static String RESULT_COMMAND = "RESULT_COMMAND";

    private Vector<String> mData;

    @Override
    public void onCreate(Bundle savedInstanceState) {

        super.onCreate(savedInstanceState);
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        setContentView(R.layout.activity_history);

        mInflater = (LayoutInflater) getSystemService(Activity.LAYOUT_INFLATER_SERVICE);
        mData = new Vector<String>();

        mMenu = new CustomMenu(this, this, getLayoutInflater());
        mMenu.setHideOnSelect(true);

        mListCommands = (ListView)findViewById(R.id.listHistoryItems);
        mListCommands.setClickable(true);
        mListCommands.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            public void onItemClick(AdapterView<?> arg0, View arg1, int position, long arg3) {
                if(position < mData.size()) {
                    String strCommand = mData.elementAt(position);
                    if(strCommand != null) {
                        getIntent().putExtra(RESULT_COMMAND, strCommand);
                        setResult(RESULT_OK, getIntent());
                        finish();
                    }
                }
            }
        });

        ImageView imageViewMenu = (ImageView)findViewById(R.id.imageViewMenu);
        imageViewMenu.setOnClickListener(new View.OnClickListener() {
            public void onClick(View arg0) {
                if (mMenu.isShowing())
                    mMenu.hide();
                else
                    mMenu.show(findViewById(R.id.layoutActionBar));
            }
        });

        ImageView imageViewBack = (ImageView)findViewById(R.id.imageViewBack);
        imageViewBack.setOnClickListener(new View.OnClickListener() {
            public void onClick(View arg0) {
                onBackPressed();
            }
        });

        LoadMenuItems();

        doHistory();
    }

    @Override
    public void onConfigurationChanged(Configuration newConfig) {

        if (mMenu.isShowing())
            mMenu.hide();

        super.onConfigurationChanged(newConfig);
    }

    public boolean onKeyDown(int keyCode, KeyEvent event) {

        if (keyCode == KeyEvent.KEYCODE_MENU) {
            if (mMenu.isShowing())
                mMenu.hide();
            else
                mMenu.show(findViewById(R.id.layoutActionBar));

            return true;
        }

        return super.onKeyDown(keyCode, event);
    }

    private void LoadMenuItems() {

        ArrayList<CustomMenuItem> menuItems = new ArrayList<CustomMenuItem>();
        CustomMenuItem cmi = new CustomMenuItem();
        cmi.setCaption(getResources().getString(R.string.stringClear));
        cmi.setImageResourceId(R.drawable.history_clear);
        cmi.setId(MENU_ITEM_CLEAR);
        menuItems.add(cmi);
        if (!mMenu.isShowing()) {
            try {
                mMenu.setMenuItems(menuItems);
            }
            catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    public void MenuItemSelectedEvent(CustomMenuItem item) {
        if (item.getId() == MENU_ITEM_CLEAR) {
            if(mData.size() > 0)
                confirmDeleteMessageBox("Are you sure you want to clear history?");
        }
    }

    private void doHistory()
    {
        SharedPreferences settings = getSharedPreferences("TinyTelnet", 0);
        // load history list
        String strHistory = settings.getString("history", "");
        if(!strHistory.equals("")) {
            if(strHistory.contains(",")) {
                String[] strAll = strHistory.split(",");
                for (String s: strAll) {
                    if(!s.equals("")) {
                        mData.add(s);
                    }
                }
            }
            else {
                mData.add(strHistory);
            }

            MyHistoryAdapter adapter = new MyHistoryAdapter(HistoryActivity.this,
                    R.layout.activity_history, R.layout.item_history, mData);
            mListCommands.setAdapter(adapter);
        }
    }

    //////////////////////////////////
    private class ViewHolder {
        private View mRow;
        private ImageView mImageViewDelete = null;
        private TextView mTextViewCommand = null;

        private ViewHolder(View row) {
            mRow = row;
        }

        private TextView getTextViewCommand() {
            if(mTextViewCommand == null)
                mTextViewCommand = (TextView)mRow.findViewById(R.id.textViewCommand);

            return mTextViewCommand;
        }

        private ImageView getImageViewDelete() {
            if(mImageViewDelete == null)
                mImageViewDelete = (ImageView)mRow.findViewById(R.id.imageViewDelete);

            return mImageViewDelete;
        }
    }

    //////////////////////////////////
    private class MyHistoryAdapter extends ArrayAdapter<String> {

        private MyHistoryAdapter(Context context, int resource, int textViewResourceId,
                                 List<String> objects) {
            super(context, resource, textViewResourceId, objects);
        }

        @Override
        public View getView(int position, View convertView, ViewGroup parent) {
            final int nIndex = position;

            ViewHolder holder;
            ImageView imageViewDelete;
            TextView textViewCommand;

            String rowData = getItem(position);
            if(null == convertView) {
                convertView = mInflater.inflate(R.layout.item_history, parent, false);
                holder = new ViewHolder(convertView);
                convertView.setTag(holder);
            }

            if(rowData == null)
                return convertView;

            holder = (ViewHolder) convertView.getTag();

            textViewCommand = holder.getTextViewCommand();
            textViewCommand.setText(rowData);

            if(position % 2 == 0)
                convertView.setBackgroundColor(0x77D5D5D5);
            else
                convertView.setBackgroundColor(0x77E0E0E0);

            imageViewDelete = holder.getImageViewDelete();
            imageViewDelete.setOnClickListener(new OnClickListener() {
                public void onClick(View v) {
                    String strCommand = getItem(nIndex);
                    remove(strCommand);

                    if(strCommand != null && !strCommand.equals("")) {
                        SharedPreferences settings = getSharedPreferences("TinyTelnet", 0);
                        SharedPreferences.Editor editor = settings.edit();

                        // load history list
                        String strHistory = settings.getString("history", "");
                        if(!strHistory.equals("")) {
                            if(strHistory.contains(",")) {
                                String[] strAll = strHistory.split(",");
                                String strNewHistory = "";
                                for (String s: strAll) {
                                    if(!s.equals("")) {
                                        if(!s.equals("") && !s.equals(strCommand)) {
                                            if(!s.equals(""))
                                                s += ",";
                                            strNewHistory += s;
                                        }
                                    }
                                }

                                if(!strNewHistory.equals(""))
                                    editor.putString("history", strNewHistory);
                            }
                            else
                                editor.putString("history", "");
                        }

                        editor.apply();
                    }
                }
            });

            return convertView;
        }
    }

    private void confirmDeleteMessageBox(String strText) {

        final Dialog dialog = new Dialog(this);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.setCancelable(true);
        dialog.setContentView(R.layout.dialog_yesno);
        Window window = dialog.getWindow();
        if(window != null)
            window.setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));

        TextView textViewMessage = (TextView)dialog.findViewById(R.id.textViewMessage);
        textViewMessage.setText(strText);

        Button buttonYes = (Button)dialog.findViewById(R.id.buttonYes);
        buttonYes.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                SharedPreferences settings = getSharedPreferences("TinyTelnet", 0);
                SharedPreferences.Editor editor = settings.edit();
                editor.putString("history", "");
                editor.apply();

                mData.clear();

                MyHistoryAdapter adapter = (MyHistoryAdapter) mListCommands.getAdapter();
                adapter.clear();

                dialog.dismiss();
            }
        });

        Button buttonNo = (Button)dialog.findViewById(R.id.buttonNo);
        buttonNo.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                dialog.dismiss();
            }
        });

        dialog.show();
    }
}